@@ -92,13 +92,19 @@ def show(ctx: Context, project_dir: str):
     "--src",
     "-s",
     help="Source tokenizer",
-    type=click.Choice(["moses", "mecab"], case_sensitive=False),
+    type=click.Choice(
+        ["moses", "mecab", "thai", "jieba"],
+        case_sensitive=False
+    ),
 )
 @click.option(
     "--tgt",
     "-t",
     help="Target tokenizer",
-    type=click.Choice(["moses", "mecab"], case_sensitive=False),
+    type=click.Choice(
+        ["moses", "mecab", "thai", "jieba"],
+        case_sensitive=False
+    ),
 )
 @click.option("--project", "-p", "project_dir")
 @click.pass_context
@@ -221,6 +227,13 @@ def processors(ctx: Context, set: str, list: bool, project_dir: str):
 )
 @click.option("--threshold", "-t", help="Threshold bpe", type=click.INT)
 @click.option("--pretrained", "-ptr", help="Model pretrained")
+@click.option(
+    "--sentencepiece", help="Use sentencepiece backend", is_flag=True
+)
+@click.option("--model_type", help="Model type for sentencepiece")
+@click.option(
+    "--vocab_size", help="Vocabulary size for sentencepiece", type=click.INT
+)
 @click.option("--project", "-p", "project_dir")
 @click.pass_context
 def bpe(
@@ -229,7 +242,10 @@ def bpe(
     num_iterations: int,
     threshold: int,
     pretrained: str,
-    project_dir: str,
+    sentencepiece: bool,
+    model_type: str,
+    vocab_size: int,
+    project_dir: str
 ):
 
     if project_dir is None:
@@ -247,7 +263,14 @@ def bpe(
 
     try:
         Config.add_bpe(
-            joint, num_iterations, threshold, pretrained, project_dir
+            joint,
+            num_iterations,
+            threshold,
+            pretrained,
+            sentencepiece,
+            model_type,
+            vocab_size,
+            project_dir
         )
     except ProjectNotFoundException as e:
         raise Exception("Command failed") from e
