@@ -6,12 +6,13 @@ from pangeamt_nlp.tokenizer.tokenizer_factory import TokenizerFactory
 from pangeamt_nlp.truecaser.truecaser import Truecaser
 from pangeamt_nlp.processor.pipeline_decoding import PipelineDecoding
 from pangeamt_nlp.bpe.bpe import BPE
+from pangeamt_nlp.bpe.sentencepiece import SentencePieceSegmenter
 from sacrebleu import corpus_bleu
 from pangeamt_nlp.translation_model.translation_model_factory import (
     TranslationModelFactory,
 )
 from pangeamt_nlp.seg import Seg
-from nteu_translation_engine.engine import Engine
+from nteu_corporate_engine.engine import Engine
 import logging
 
 
@@ -25,7 +26,6 @@ class EvalStage(BaseStage):
     async def run(self, gpu: int = None, step: int = None):
         project = self.workflow.project
         project_dir = project.config.project_dir
-        config = project.config
 
         workflow_dir = self.workflow.get_dir(project_dir)
         self.stage_dir = os.path.join(workflow_dir, EvalStage.DIR)
@@ -33,9 +33,6 @@ class EvalStage(BaseStage):
         if not os.path.isdir(self.stage_dir):
             os.mkdir(self.stage_dir)
 
-        name = config.translation_model["name"]
-        args = config.translation_model["args_decoding"]
-        translation_model = TranslationModelFactory.get_class(name)
         model_dir = os.path.join(workflow_dir, "03_trained")
         model_path = None
 
@@ -48,82 +45,19 @@ class EvalStage(BaseStage):
             raise Exception("No model found.")
 
         self._prepare_dir = os.path.join(workflow_dir, "02_prepared")
-        tokenizer = self._init_tokenizer()
-        truecaser = Truecaser()
-        pipeline = self._init_pipeline()
 
-        src_path = os.path.join(self._prepare_dir, "04_bpe", "test_src.txt")
+        src_path = os.path.join(self._prepare_dir, "01_raw", "test_src.txt")
         tgt_path = os.path.join(self._prepare_dir, "01_raw", "test_tgt.txt")
         out_path = os.path.join(self.stage_dir, "test_output.txt")
 
-        if gpu is not None and gpu > -1:
-            args["gpu"] = gpu
-
-        logging.info("Loading model..")
-
-        decoding_model = translation_model(model_path, **args)
-
-        logging.info("Model loaded")
-
-        with open(src_path, "r") as src_file, open(out_path, "w") as out_file:
-            batch = []
-            segs = []
-            with click.progressbar(
-                enumerate(src_file),
-                length=2000,
-                label="Evaluating: ",
-            ) as bar:
-                for i, src_line in bar:
-                    batch.append(src_line)
-                    segs.append(Seg(src_line))
-                    if (i + 1) % 20 == 0:
-                        for translation, seg in zip(
-                            decoding_model.translate(batch), segs
-                        ):
-                            seg.tgt_raw = translation
-                            translation = BPE.undo(translation)
-                            translation = tokenizer.detokenize(
-                                translation.split(" ")
-                            )
-                            translation = truecaser.detruecase(translation)
-                            seg.tgt = translation
-                            pipeline.process_tgt(seg)
-                            out_file.write(seg.tgt + "\n")
-                        batch = []
-                        segs = []
-
-        decoding_model = None
-
-        with open(out_path, "r") as sys_file, open(tgt_path, "r") as ref_file:
-            final_score = corpus_bleu(sys_file, [ref_file]).score
-
-        logging.info("Bleu score: ", final_score)
+        final_score = await self.eval_files(
+            gpu, step, src_path, tgt_path, out_path, None, False
+        )
 
         return {
             "Score": final_score
         }
 
-    def _init_pipeline(self) -> PipelineDecoding:
-        project = self.workflow.project
-        config = project.config
-        src_lang = config.src_lang
-        tgt_lang = config.tgt_lang
-        processors = config.processors
-
-        return PipelineDecoding.create_from_dict(
-            src_lang, tgt_lang, processors
-        )
-
-    def _init_tokenizer(self):
-        project = self.workflow.project
-        tgt_tok_name = project.config.tokenizer["tgt"]
-
-        tgt_tokenizer = TokenizerFactory.new(
-            project.config.tgt_lang, tgt_tok_name
-        )
-
-        return tgt_tokenizer
-
     async def eval_files(
         self,
         gpu: int,
@@ -161,7 +95,8 @@ class EvalStage(BaseStage):
             "model_path": model_path,
             "bpe": os.path.join(self._prepare_dir, "bpe_model"),
             "truecaser": os.path.join(self._prepare_dir, "truecase_model"),
-            "gpu": True if gpu is not None else False
+            "gpu": True if gpu is not None else False,
+            "sentencepiece": config["bpe"].get("sentencepiece")
         }
 
         if log_file is not None:
@@ -201,4 +136,5 @@ class EvalStage(BaseStage):
         with open(out_path, "r") as sys_file, open(ref_path, "r") as ref_file:
             final_score = corpus_bleu(sys_file, [ref_file]).score
 
-        logging.info("Score: ", final_score)
+        logging.info("Score: " + str(final_score))
+        return final_score
