@@ -13,6 +13,7 @@ from pangeamt_tea.project.workflow.stage.base_stage import BaseStage
 from pangeamt_tea.project.workflow.stage.stage_factory import StageFactory
 from pangeamt_nlp.truecaser.truecaser import Truecaser
 from pangeamt_nlp.bpe.bpe import BPE
+from pangeamt_nlp.bpe.sentencepiece import SentencePieceSegmenter
 from pangeamt_nlp.data_batcher.batcher import batch
 
 
@@ -158,6 +159,7 @@ class PrepareStage(BaseStage):
         all_lines_shuf = os.path.join(dir, "all_lines_shuf.txt")
         temp_path = tempfile.mkstemp()[1]
         all_lines = open(all_lines_path, "w+")
+        special_order_src_langs = ["th"]
 
         # Reads the cleaned files and joins them in a file
         with open(os.path.join(cleaned_dir, "data_src.txt"), "r") as src_file:
@@ -165,10 +167,19 @@ class PrepareStage(BaseStage):
                 os.path.join(cleaned_dir, "data_tgt.txt"), "r"
             ) as tgt_file:
                 for src_line, tgt_line in zip(src_file, tgt_file):
-                    # Writes src_line###SEPARATOR###tgt_line
-                    all_lines.write(
-                        f"{src_line[:-1]}###SEPARATOR###{tgt_line}"
-                    )
+                    if project.config.src_lang in special_order_src_langs:
+                        line_to_write = (
+                            f"{tgt_line.strip()}"
+                            "###SEPARATOR###"
+                            f"{src_line.strip()}\n"
+                        )
+                    else:
+                        line_to_write = (
+                            f"{src_line.strip()}"
+                            "###SEPARATOR###"
+                            f"{tgt_line.strip()}\n"
+                        )
+                    all_lines.write(line_to_write)
 
         all_lines.close()
 
@@ -212,21 +223,25 @@ class PrepareStage(BaseStage):
         with open(all_lines_shuf, "r") as all_lines:
             for i, line in enumerate(all_lines):
                 line = line.split("###SEPARATOR###")
-                src = line[0]
-                tgt = line[1]
+                if project.config.src_lang in special_order_src_langs:
+                    tgt = line[0].strip() + "\n"
+                    src = line[1].strip() + "\n"
+                else:
+                    src = line[0].strip() + "\n"
+                    tgt = line[1].strip() + "\n"
 
                 # Take first 2000 lines for dev
                 if i < 2000:
                     # Only src need to be appended a new line
-                    dev_src.write(src + "\n")
+                    dev_src.write(src)
                     dev_tgt.write(tgt)
                 # Take second 2000 lines for test
                 elif i < 4000:
-                    test_src.write(src + "\n")
+                    test_src.write(src)
                     test_tgt.write(tgt)
                 # Take the rest of the lines for train
                 else:
-                    train_src.write(src + "\n")
+                    train_src.write(src)
                     train_tgt.write(tgt)
 
         self.NUM_LINES = i
@@ -291,7 +306,7 @@ class PrepareStage(BaseStage):
                 in_path = os.path.join(prev_dir, file)
                 out_path = os.path.join(dir, file)
                 with open(in_path, "r", encoding="utf-8") as in_file:
-                    with open(out_path, "w+", encoding="utf-8") as out_file:
+                    with open(out_path, "w+") as out_file:
                         with closing(Pool(max_workers)) as pool:
                             with click.progressbar(
                                 pool.imap(
@@ -428,6 +443,59 @@ class PrepareStage(BaseStage):
 
         # Directory of the previous stage
         prev_dir = os.path.join(self.stage_dir, "03_truecased")
+
+        if self.workflow.project.config.bpe.get("sentencepiece"):
+            src_segmenter, tgt_segmenter = self.sentencepiece_segmenter(
+                prev_dir
+            )
+        else:
+            src_segmenter, tgt_segmenter = self.bpe_segmenter(prev_dir)
+
+        # Pair src and tgt files with the corresponding BPE objects
+        files = (
+            (
+                [
+                    ("train_src.txt", self.NUM_LINES),
+                    ("test_src.txt", 2000),
+                    ("dev_src.txt", 2000),
+                ],
+                src_segmenter,
+            ),
+            (
+                [
+                    ("train_tgt.txt", self.NUM_LINES),
+                    ("test_tgt.txt", 2000),
+                    ("dev_tgt.txt", 2000),
+                ],
+                tgt_segmenter,
+            ),
+        )
+        logging.info("Starting to apply bpe")
+        for pair in files:
+            bpe = pair[1]
+            if bpe:
+                for file, size in pair[0]:
+                    in_path = os.path.join(prev_dir, file)
+                    out_path = os.path.join(dir, file)
+                    with open(in_path, "r", encoding="utf-8") as in_file:
+                        with open(
+                            out_path, "w+", encoding="utf-8"
+                        ) as out_file:
+                            with closing(Pool(max_workers)) as pool:
+                                with click.progressbar(
+                                    pool.imap(
+                                        lambda x: bpe.apply(x),
+                                        in_file,
+                                        chunksize=self.CHUNKSIZE,
+                                    ),
+                                    length=size,
+                                    label=f"Applying bpe: {file}",
+                                ) as bar:
+                                    for bpe_line in bar:
+                                        out_file.write(bpe_line)
+        logging.info("Finished applying bpe")
+
+    def bpe_segmenter(self, prev_dir):
         # BPE joint and threshold parameter from config
         joint = self.workflow.project.config.bpe["joint"]
         threshold = self.workflow.project.config.bpe["threshold"]
@@ -460,50 +528,7 @@ class PrepareStage(BaseStage):
                 os.path.join(model_dir, "tgt_codes.txt"),
                 bpe_threshold=threshold,
             )
-
-        # Pair src and tgt files with the corresponding BPE objects
-        files = (
-            (
-                [
-                    ("train_src.txt", self.NUM_LINES),
-                    ("test_src.txt", 2000),
-                    ("dev_src.txt", 2000),
-                ],
-                src_bpe,
-            ),
-            (
-                [
-                    ("train_tgt.txt", self.NUM_LINES),
-                    ("test_tgt.txt", 2000),
-                    ("dev_tgt.txt", 2000),
-                ],
-                tgt_bpe,
-            ),
-        )
-        logging.info("Starting to apply bpe")
-        for pair in files:
-            bpe = pair[1]
-            if bpe:
-                for file, size in pair[0]:
-                    in_path = os.path.join(prev_dir, file)
-                    out_path = os.path.join(dir, file)
-                    with open(in_path, "r", encoding="utf-8") as in_file:
-                        with open(
-                            out_path, "w+", encoding="utf-8"
-                        ) as out_file:
-                            with closing(Pool(max_workers)) as pool:
-                                with click.progressbar(
-                                    pool.imap(
-                                        lambda x: bpe.apply(x),
-                                        in_file,
-                                        chunksize=self.CHUNKSIZE,
-                                    ),
-                                    length=size,
-                                    label=f"Applying bpe: {file}",
-                                ) as bar:
-                                    for bpe_line in bar:
-                                        out_file.write(bpe_line)
-        logging.info("Finished applying bpe")
+        return src_bpe, tgt_bpe
 
     # Create BPE model
     def create_bpe_model(self, prev_dir, joint):
@@ -550,7 +575,55 @@ class PrepareStage(BaseStage):
                 tgt_model_path = os.path.join(dir, "tgt_codes.txt")
                 BPE.learn(src_input_path, src_model_path, iterations)
                 BPE.learn(tgt_input_path, tgt_model_path, iterations)
+        return dir
 
+    def sentencepiece_segmenter(self, prev_dir):
+        vocab_size = (
+            8000
+            if self.workflow.project.config.bpe["vocab_size"] is None
+            else self.workflow.project.config.bpe["vocab_size"]
+        )
+        model_type = (
+            "unigram"
+            if self.workflow.project.config.bpe["model_type"] is None
+            else self.workflow.project.config.bpe["model_type"]
+        )
+        logging.info("Training sentencepiece models if enabled...")
+        model_dir = self.create_sentencepiece_model(
+            prev_dir, vocab_size, model_type
+        )
+        logging.info("Finished training sentencepiece model")
+        # If joint, train the bpe with all the corpus, else, train with the
+        # corpus split by language
+        src_sentencepiece = SentencePieceSegmenter(
+            os.path.join(model_dir, "src_codes.model")
+        )
+        tgt_sentencepiece = SentencePieceSegmenter(
+            os.path.join(model_dir, "tgt_codes.model")
+        )
+        return src_sentencepiece, tgt_sentencepiece
+
+    # Create Sentencepiece model
+    def create_sentencepiece_model(self, prev_dir, vocab_size, model_type):
+        config = self.workflow.project.config
+        dir = os.path.join(self.stage_dir, "bpe_model")
+        src_input_path = os.path.join(prev_dir, "train_src.txt")
+        tgt_input_path = os.path.join(prev_dir, "train_tgt.txt")
+
+        try:
+            os.mkdir(dir)
+        except FileExistsError:
+            shutil.rmtree(dir)
+            os.mkdir(dir)
+
+        src_model_path = os.path.join(dir, "src_codes")
+        tgt_model_path = os.path.join(dir, "tgt_codes")
+        SentencePieceSegmenter.learn(
+            src_input_path, src_model_path, model_type, vocab_size
+        )
+        SentencePieceSegmenter.learn(
+            tgt_input_path, tgt_model_path, model_type, vocab_size
+        )
         return dir
 
     # Split the corpus in shards
